// =======================================================================================
//
//  orbitconfig.cpp
//
//  Screen Saver Module for BeOS - configuration class
//
// =======================================================================================

#include <Window.h>
#include <StringView.h>
#include <TabView.h>
#include "orbitconfig.h"

enum {
	ORBMSG_PLANETSIZE = 'opsz',
	ORBMSG_ALPHAVAL   = 'opal',
	ORBMSG_PLANETMOT  = 'opmt',

	ORBMSG_SATSIZE    = 'ossz',
	ORBMSG_SATNUM     = 'osnm',
	ORBMSG_SATMOT     = 'osmt',

	ORBMSG_FOGCHECK   = 'ofog',
	ORBMSG_BACKGROUND = 'obkg'
};

BView * AddNewTab(BTabView *pTabView, const char *pLabel);

// ---------------------------------------------------------------------------------------
//  COrbitConfig Member Functions
// ---------------------------------------------------------------------------------------

COrbitConfig::COrbitConfig(BRect rect, OrbitSettings *pSetIn)
	: BView(rect, "", B_FOLLOW_NONE, B_WILL_DRAW)
{
	if(!pSetIn)
		return;

	pSettings = pSetIn;

	// configure the controls

	pCtlAlpha = 0; pCtlFog = 0;

	BTabView *pTabView = new BTabView(rect, "tab view");
	AddChild(pTabView);

	BView *pSettingsTab = AddNewTab(pTabView, "Settings");
	if(pSettingsTab)
	{
		// 2 views -- one for planet settings, the other for satellite settings

		BRect rect(pSettingsTab->Bounds());

		rect.right = (rect.right / 2) - 5;
		rect.bottom -= 20;

		// The Planet Settings View

		BView *pPlanetView = new BView(rect, 0, B_FOLLOW_ALL, B_WILL_DRAW);
		if(pPlanetView)
		{
			pPlanetView->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
			pSettingsTab->AddChild(pPlanetView);

			pCtlPSize = new BSlider(
				BRect(0, 5, 100, 50), "CtlPSize", "Planet Size", new BMessage(ORBMSG_PLANETSIZE),
				1, 20, B_BLOCK_THUMB); 
			if(pCtlPSize)
			{
				pCtlPSize->SetHashMarks(B_HASH_MARKS_BOTTOM);
				pCtlPSize->SetLimitLabels("1", "20");
				pCtlPSize->SetHashMarkCount(10);
				pCtlPSize->SetValue(pSettings->nPlanetSize);
				pPlanetView->AddChild(pCtlPSize);
			}

			pCtlAlpha = new BSlider(
				BRect(0, 60, 100, 105), "CtlAlpha", "Planet Opacity", new BMessage(ORBMSG_ALPHAVAL),
				0, 100, B_BLOCK_THUMB); 
			if(pCtlAlpha)
			{
				pCtlAlpha->SetHashMarks(B_HASH_MARKS_BOTTOM);
				pCtlAlpha->SetLimitLabels("0", "100");
				pCtlAlpha->SetHashMarkCount(10);
				pCtlAlpha->SetValue(pSettings->fPlanetAlpha * 100);
				pPlanetView->AddChild(pCtlAlpha);
			}

			// The "Planet Motion" Radio Buttons

			pPlanetView->AddChild(new BStringView(
				BRect(0, 120, 100, 132), B_EMPTY_STRING, "Planet Motion:"));

			pCtlMot0 = new BRadioButton(
				BRect(5, 135, 100, 132), "CtlMot0", "Bounce", new BMessage(ORBMSG_PLANETMOT));
			if(pCtlMot0)
			{
				pCtlMot0->SetValue((pSettings->nMotionType == MOTION_BOUNCE) ? B_CONTROL_ON : B_CONTROL_OFF);
				pPlanetView->AddChild(pCtlMot0);
			} 

			pCtlMot1 = new BRadioButton(
				BRect(5, 155, 100, 152), "CtlMot1", "Orbit", new BMessage(ORBMSG_PLANETMOT));
			if(pCtlMot1)
			{
				pCtlMot1->SetValue((pSettings->nMotionType == MOTION_ORBIT) ? B_CONTROL_ON : B_CONTROL_OFF);
				pPlanetView->AddChild(pCtlMot1);
			}
		}

		// The Satellite Settings View

		rect = pSettingsTab->Bounds();
		rect.left = (rect.right / 2) + 5;
		rect.bottom -= 20;

		BView *pSatView = new BView(rect, 0, B_FOLLOW_ALL, B_WILL_DRAW);
		if(pSatView)
		{
			pSatView->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
			pSettingsTab->AddChild(pSatView);

			pCtlSSize = new BSlider(
				BRect(0, 5, 100, 50), "CtlSSize", "Satellite Size", new BMessage(ORBMSG_SATSIZE),
				1, 10, B_BLOCK_THUMB); 
			if(pCtlSSize)
			{
				pCtlSSize->SetHashMarks(B_HASH_MARKS_BOTTOM);
				pCtlSSize->SetLimitLabels("1", "10");
				pCtlSSize->SetHashMarkCount(10);
				pCtlSSize->SetValue(pSettings->nSatSize);
				pSatView->AddChild(pCtlSSize);
			}

			pCtlSNum = new BSlider(
				BRect(0, 60, 100, 105), "CtlSNum", "# of Satellites",  new BMessage(ORBMSG_SATNUM),
				1, 10, B_BLOCK_THUMB); 
			if(pCtlSNum)
			{
				pCtlSNum->SetHashMarks(B_HASH_MARKS_BOTTOM);
				pCtlSNum->SetLimitLabels("1", "10");
				pCtlSNum->SetHashMarkCount(10);
				pCtlSNum->SetValue(pSettings->nSatellites);
				pSatView->AddChild(pCtlSNum);
			}

			pSatView->AddChild(new BStringView(
				BRect(0, 120, 100, 132), B_EMPTY_STRING, "Satellite Motion:"));

			pCtlOrb0 = new BRadioButton(
				BRect(5, 135, 100, 132), "CtlOrb0", "Spherical", new BMessage(ORBMSG_SATMOT));
			if(pCtlOrb0)
			{
				pCtlOrb0->SetValue((pSettings->nOrbitType == MOTION_SPHERICAL) ? B_CONTROL_ON : B_CONTROL_OFF);
				pSatView->AddChild(pCtlOrb0);
			} 

			pCtlOrb1 = new BRadioButton(
				BRect(5, 155, 100, 152), "CtlOrb1", "Planar", new BMessage(ORBMSG_SATMOT));
			if(pCtlMot1)
			{
				pCtlOrb1->SetValue((pSettings->nOrbitType == MOTION_PLANAR) ? B_CONTROL_ON : B_CONTROL_OFF);
				pSatView->AddChild(pCtlOrb1);
			}
		}

		// miscellaneous controls

		pCtlFog = new BCheckBox(
			BRect(0, 190, 100, 220), "CtlFog", "Enable Fog", new BMessage(ORBMSG_FOGCHECK));
		if(pCtlFog)
		{
			pCtlFog->SetValue(pSettings->bFog ? B_CONTROL_ON : B_CONTROL_OFF);
			pSettingsTab->AddChild(pCtlFog);
		}
/*
	No "background" checkbox yet.  This is for 1.2.

		pCtlBG = new BCheckBox(
			BRect(110, 190, 220, 220), "CtlBG", "Background", new BMessage(ORBMSG_BACKGROUND));
		if(pCtlBG)
		{
			pCtlBG->SetValue(pSettings->bBackground ? B_CONTROL_ON : B_CONTROL_OFF);
			pSettingsTab->AddChild(pCtlBG);
		}
*/
	}

	BView * pAboutTab = AddNewTab(pTabView, "About");
	if(pAboutTab)
	{
		pAboutTab->AddChild(new BStringView(
			BRect(10, 10, 240, 22), B_EMPTY_STRING, APPNAME));
		pAboutTab->AddChild(new BStringView(
			BRect(10, 30, 240, 42), B_EMPTY_STRING, "by Eron Hennessey"));
		pAboutTab->AddChild(new BStringView(
			BRect(10, 90, 240, 102), B_EMPTY_STRING, "email: eronh@nwlink.com"));
	}
}

// ---------------------------------------------------------------------------------------

void COrbitConfig::AttachedToWindow()
{
	SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));

	pCtlPSize->SetTarget(this);
	pCtlAlpha->SetTarget(this);
	pCtlMot0->SetTarget(this);
	pCtlMot1->SetTarget(this);

	pCtlSSize->SetTarget(this);
	pCtlSNum->SetTarget(this);
	pCtlOrb0->SetTarget(this);
	pCtlOrb1->SetTarget(this);

	pCtlFog->SetTarget(this);
//	pCtlBG->SetTarget(this);

	nWindowFlags = Window()->Flags();
	Window()->SetFlags(nWindowFlags | B_ASYNCHRONOUS_CONTROLS);
}

// --------------------------------------------------------------------------------------

void COrbitConfig::DetachedFromWindow()
{
	Window()->SetFlags(nWindowFlags);
}

// --------------------------------------------------------------------------------------

void COrbitConfig::MessageReceived(BMessage *pMsg)
{
	switch(pMsg->what)
	{
		case ORBMSG_PLANETSIZE:
			pSettings->nPlanetSize = pCtlPSize->Value();
			break;
		case ORBMSG_ALPHAVAL:
			pSettings->fPlanetAlpha = (pCtlAlpha->Value() / 100.0f);
			break;
		case ORBMSG_PLANETMOT:
			pSettings->nMotionType = ((pCtlMot0->Value() == B_CONTROL_ON) ? MOTION_BOUNCE : MOTION_ORBIT); 
			break;

		case ORBMSG_SATSIZE:
			pSettings->nSatSize = pCtlSSize->Value();
			break;
		case ORBMSG_SATNUM:
			pSettings->nSatellites = pCtlSNum->Value();
			break;
		case ORBMSG_SATMOT:
			pSettings->nOrbitType = ((pCtlOrb0->Value() == B_CONTROL_ON) ? MOTION_SPHERICAL : MOTION_PLANAR);
			break;

		case ORBMSG_FOGCHECK:
			pSettings->bFog = (pCtlFog->Value() == B_CONTROL_ON);
			break;
		case ORBMSG_BACKGROUND:
			pSettings->bBackground = (pCtlBG->Value() == B_CONTROL_ON);
			break;

		default:
			BView::MessageReceived(pMsg);
			break;
	}
}

// --------------------------------------------------------------------------------------

BView * AddNewTab(BTabView *pTabView, const char *pLabel)
{
	BRect rect;
	rect = pTabView->Bounds();
	rect.InsetBy(5,5);
	rect.bottom -= pTabView->TabHeight();

	BView *pView = new BView(rect, 0, B_FOLLOW_ALL, B_WILL_DRAW);
	BTab  *pTab = new BTab();
	pTabView->AddTab(pView, pTab);

	pView->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	pTab->SetLabel(pLabel);

	return pView;
}

