// =======================================================================================
//
//  orbitview.cpp
//
//  Screen Saver Module for BeOS - view class
//
// =======================================================================================

#include <math.h>
#include "orbitview.h"
#include "rnumber.h"

const float MINX = -90.0f;
const float MAXX = 90.0f;
const float MINY = -70.0f;
const float MAXY = 70.0f;

void Normalize(float *pVec);
void OrthoVec(float *pVecIn, float *pVecOut);

// ---------------------------------------------------------------------------------------
//  COrbitView Member Functions
// ---------------------------------------------------------------------------------------

COrbitView::COrbitView(BRect rect, char *name, ulong mode, ulong opt, OrbitSettings *pSetIn)
	: BGLView(rect, name, mode, 0, opt)
{
	if(!pSetIn)
		return;

	Randomize();

	pSettings = pSetIn;
	pQuadric = 0;

	SetupObjects();
	SetupScene(rect);
}

// ---------------------------------------------------------------------------------------

void COrbitView::SetupScene(BRect &rect)
{
	float planet_color[]  = { 1.0f, 1.0f, 1.0f, 1.0f };
	float amb_color[]     = { 0.1f, 0.1f, 0.1f, 1.0f };
	float white_color[]   = { 1.0f, 1.0f, 1.0f, 1.0f };
	float black_color[]   = { 0.0f, 0.0f, 0.0f, 0.0f };
	float mat_shin        = 60.0f;
	const float near      = 20.0f;
	const float far       = 500.0f; 

	planet_color[3] = pSettings->fPlanetAlpha;

	LockGL();

	glLightfv(GL_LIGHT0, GL_AMBIENT, amb_color);
	glEnable(GL_LIGHT0);

	// set up the initial projection and view matrices.

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60, (float)(rect.Width() / rect.Height()), near, far);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(
		0.0f, 0.0f, -150.0f, // from
		0.0f, 0.0f, 0.0f,    // at
		0.0f, 1.0f, 0.0f);   // up

	// set some gl properties.
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_LIGHTING);
	glEnable(GL_DITHER);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	// fog properties
	if(pSettings->bFog)
	{
		glEnable(GL_FOG);
		glFogi(GL_FOG_MODE, GL_LINEAR);
		glFogfv(GL_FOG_COLOR, black_color);
		glFogf(GL_FOG_DENSITY, 0.35f);
		glFogf(GL_FOG_START, near);
		glFogf(GL_FOG_END, far);
	}
	
	glMaterialfv(GL_FRONT, GL_SPECULAR, white_color);
	glMaterialfv(GL_FRONT, GL_SHININESS, &mat_shin);

	// create display lists to *hopefully* improve mode-switching performance

	const float light_pos[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
	float ovec[3];

	OrthoVec(PlanetData.spinaxis, ovec);

	for(int i = 0; i < pSettings->nSatellites; i++)
	{
		if(pSettings->nOrbitType == MOTION_SPHERICAL)
			OrthoVec(vSatData[i].orbitaxis, ovec);

		vSatData[i].nList = glGenLists(1);
		glNewList(vSatData[i].nList, GL_COMPILE);
			glTranslatef(ovec[0]*vSatData[i].dist, ovec[1]*vSatData[i].dist, ovec[2]*vSatData[i].dist);
			glLightfv(GL_LIGHT0 + i, GL_DIFFUSE, vSatData[i].color);
			glLightfv(GL_LIGHT0 + i, GL_SPECULAR, white_color);
			glLightfv(GL_LIGHT0 + i, GL_POSITION, light_pos);
			glEnable(GL_LIGHT0 + i);
			glMaterialfv(GL_FRONT, GL_DIFFUSE, vSatData[i].color);
			glMaterialfv(GL_FRONT, GL_AMBIENT, vSatData[i].color);
			gluSphere(pQuadric, (float)pSettings->nSatSize, 16, 16);
		glEndList();
	}

	PlanetData.nList = glGenLists(1);
	glNewList(PlanetData.nList, GL_COMPILE);
		glMaterialfv(GL_FRONT, GL_DIFFUSE, planet_color);
		glMaterialfv(GL_FRONT, GL_AMBIENT, planet_color);
		glEnable(GL_BLEND);
		gluSphere(pQuadric, (float)pSettings->nPlanetSize, 32, 32);
		glDisable(GL_BLEND);
	glEndList();	

	UnlockGL();
}

// ---------------------------------------------------------------------------------------

void COrbitView::SetupObjects()
{
	if(!(pQuadric = gluNewQuadric()))
		return;

	// setup planet data

	PlanetData.spinamt = 1.0f;
	PlanetData.spinaxis[0] = (float)GetRandomNumber(-1000, 1000);
	PlanetData.spinaxis[1] = (float)GetRandomNumber(-1000, 1000);
	PlanetData.spinaxis[2] = (float)GetRandomNumber(-1000, 1000);
	Normalize(PlanetData.spinaxis);

	PlanetData.pos[0] = 0.0f; 
	PlanetData.pos[1] = 0.0f;
	PlanetData.pos[2] = 0.0f;
	PlanetData.dir[0] = (float)GetRandomNumber(-1000, 1000);
	PlanetData.dir[1] = (float)GetRandomNumber(-1000, 1000);
	PlanetData.dir[2] = 0.0f;
	Normalize(PlanetData.dir);

	PlanetData.orbitamt = (float)GetRandomNumber(0, 359);
	PlanetData.orbitaxis[0] = (float)GetRandomNumber(-1000, 1000);
	PlanetData.orbitaxis[1] = (float)GetRandomNumber(-1000, 1000);
	PlanetData.orbitaxis[2] = (float)GetRandomNumber(-1000, 1000);
	Normalize(PlanetData.orbitaxis);

	// setup satellite data
	float satdist = 30.0f;

	for(int i = 0; i < pSettings->nSatellites; i++)
	{
		satdata sd;

		sd.color[0] = (float)GetRandomNumber(0,1000);
		sd.color[1] = (float)GetRandomNumber(0,1000);
		sd.color[2] = (float)GetRandomNumber(0,1000);
		sd.color[3] = 1.0f;
		Normalize(sd.color);
		
		sd.dist = satdist;

		if(pSettings->nOrbitType == MOTION_SPHERICAL)
		{
			sd.orbitspd = (float)GetRandomNumber(200, 1000) / 200.0f;
			sd.orbitaxis[0] = (float)GetRandomNumber(-1000, 1000);
			sd.orbitaxis[1] = (float)GetRandomNumber(-1000, 1000);
			sd.orbitaxis[2] = (float)GetRandomNumber(-1000, 1000);
			Normalize(sd.orbitaxis);
		}
		else // MOTION_PLANAR
		{
			sd.orbitspd = sqrt((240.0f * pSettings->nPlanetSize) / (satdist * satdist));
			sd.orbitaxis[0] = PlanetData.spinaxis[0];
			sd.orbitaxis[1] = PlanetData.spinaxis[1];
			sd.orbitaxis[2] = PlanetData.spinaxis[2]; 
			satdist += 10.0f;
		}

		sd.orbitamt = (float)GetRandomNumber(0, 359);

		vSatData.push_back(sd);
	}	

}

// ---------------------------------------------------------------------------------------

COrbitView::~COrbitView()
{
	if(pQuadric) gluDeleteQuadric(pQuadric);
}

// ---------------------------------------------------------------------------------------

void COrbitView::AttachedToWindow()
{
	BGLView::AttachedToWindow();
}

// ---------------------------------------------------------------------------------------

void COrbitView::DrawFrame()
{
	LockGL();
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	if((PlanetData.spinamt += 1.0f) >= 360.0f)
		PlanetData.spinamt == 0.0f;

	glPushMatrix();
	if(pSettings->nMotionType == MOTION_BOUNCE)
	{
		if((PlanetData.pos[0] < MINX) || (PlanetData.pos[0] > MAXX))
			PlanetData.dir[0] = -PlanetData.dir[0];
		if((PlanetData.pos[1] < MINY) || (PlanetData.pos[1] > MAXY))
			PlanetData.dir[1] = -PlanetData.dir[1];

		PlanetData.pos[0] += PlanetData.dir[0];
		PlanetData.pos[1] += PlanetData.dir[1];
		glTranslatef(PlanetData.pos[0], PlanetData.pos[1], PlanetData.pos[2]);
	}
	else // MOTION_ORBIT
	{
		if((PlanetData.orbitamt += 1.0f) >= 360.0f)
			PlanetData.orbitamt == 0.0f;
		glRotatef(PlanetData.orbitamt, PlanetData.orbitaxis[0], PlanetData.orbitaxis[1], PlanetData.orbitaxis[2]);
		glTranslatef(0.0f, 0.0f, 50.0f);
	}

	glRotatef(PlanetData.spinamt, PlanetData.spinaxis[0], PlanetData.spinaxis[1], PlanetData.spinaxis[2]);

	// update the satellite orbits

	for(int i = 0; i < pSettings->nSatellites; i++)
	{
		if((vSatData[i].orbitamt += vSatData[i].orbitspd) >= 360.0f)
			vSatData[i].orbitamt = (0.0f + (vSatData[i].orbitspd - 360.0f));

		glPushMatrix();
		glRotatef(vSatData[i].orbitamt, vSatData[i].orbitaxis[0], vSatData[i].orbitaxis[1], vSatData[i].orbitaxis[2]);
		glCallList(vSatData[i].nList);
		glPopMatrix();
	}

	glCallList(PlanetData.nList);
	glPopMatrix();

	SwapBuffers();
	UnlockGL();
}

// ---------------------------------------------------------------------------------------

void Normalize(float *pVec)
{
	float fSize = sqrt(pVec[0]*pVec[0] + pVec[1]*pVec[1] + pVec[2]*pVec[2]);

	if(fSize)
	{
		pVec[0] /= fSize;
		pVec[1] /= fSize;
		pVec[2] /= fSize;
	}
}

// ---------------------------------------------------------------------------------------

void OrthoVec(float *a, float *c)
{
	float b[3] = { 0.0f, 0.0f, 1.0f };

	c[0] = a[1]*b[2] - a[2]*b[1];
	c[1] = a[2]*b[0] - a[0]*b[2];
	c[2] = a[0]*b[1] - a[1]*b[0];	
	Normalize(c);
}

//========================================================================================
// EOF
